/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#include "gfs.h"

#define pv(struct, member, fmt) printk("  "#member" = "fmt"\n", struct->member);

#define CPIN_08(s1, s2, member, count) {memcpy((s1->member), (s2->member), (count));}
#define CPOUT_08(s1, s2, member, count) {memcpy((s2->member), (s1->member), (count));}
#define CPIN_16(s1, s2, member) {(s1->member) = gfs16_to_cpu((s2->member));}
#define CPOUT_16(s1, s2, member) {(s2->member) = cpu_to_gfs16((s1->member));}
#define CPIN_32(s1, s2, member) {(s1->member) = gfs32_to_cpu((s2->member));}
#define CPOUT_32(s1, s2, member) {(s2->member) = cpu_to_gfs32((s1->member));}
#define CPIN_64(s1, s2, member) {(s1->member) = gfs64_to_cpu((s2->member));}
#define CPOUT_64(s1, s2, member) {(s2->member) = cpu_to_gfs64((s1->member));}

/**
 * gfs_rgrp_lvb_in - Read in rgrp data
 * @rb: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_rgrp_lvb_in(struct gfs_rgrp_lvb *rb, char *lvb)
{
	struct gfs_rgrp_lvb *str = (struct gfs_rgrp_lvb *) lvb;

	CPIN_32(rb, str, rb_magic);
	CPIN_32(rb, str, rb_free);
	CPIN_32(rb, str, rb_useddi);
	CPIN_32(rb, str, rb_freedi);
	CPIN_32(rb, str, rb_usedmeta);
	CPIN_32(rb, str, rb_freemeta);
	CPIN_16(rb, str, rb_owner);

}

/**
 * gfs_rgrp_lvb_out - Write out rgrp data
 * @rb: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_rgrp_lvb_out(struct gfs_rgrp_lvb *rb, char *lvb)
{
	struct gfs_rgrp_lvb *str = (struct gfs_rgrp_lvb *) lvb;

	CPOUT_32(rb, str, rb_magic);
	CPOUT_32(rb, str, rb_free);
	CPOUT_32(rb, str, rb_useddi);
	CPOUT_32(rb, str, rb_freedi);
	CPOUT_32(rb, str, rb_usedmeta);
	CPOUT_32(rb, str, rb_freemeta);
	CPOUT_16(rb, str, rb_owner);

}

/**
 * gfs_rgrp_lvb_print - Print out rgrp data
 * @rb: the cpu-order structure
 * @console - TRUE if this should be printed to the console,
 *            FALSE if it should be just printed to the incore debug
 *            buffer
 */

void
gfs_rgrp_lvb_print(struct gfs_rgrp_lvb *rb, int console)
{

	pv(rb, rb_magic, "%u");
	pv(rb, rb_free, "%u");
	pv(rb, rb_useddi, "%u");
	pv(rb, rb_freedi, "%u");
	pv(rb, rb_usedmeta, "%u");
	pv(rb, rb_freemeta, "%u");
	pv(rb, rb_owner, "%u");

}

/**
 * gfs_plock_phead_in - Read in plock head data
 * @ph: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_plock_phead_in(struct gfs_plock_phead *ph, char *lvb)
{
	struct gfs_plock_phead *str = (struct gfs_plock_phead *) lvb;

	CPIN_32(ph, str, ph_magic);
	CPIN_32(ph, str, ph_gid);

}

/**
 * gfs_plock_phead_out - Write out plock head data
 * @ph: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_plock_phead_out(struct gfs_plock_phead *ph, char *lvb)
{
	struct gfs_plock_phead *str = (struct gfs_plock_phead *) lvb;

	CPOUT_32(ph, str, ph_magic);
	CPOUT_32(ph, str, ph_gid);

}

/**
 * gfs_plock_phead_print - print out phead data
 * @ph: the cpu-order structure
 * @console - TRUE if this should be printed to the console,
 *            FALSE if it should be just printed to the incore debug
 *            buffer
 */

void
gfs_plock_phead_print(struct gfs_plock_phead *ph, int console)
{

	pv(ph, ph_magic, "%x");
	pv(ph, ph_gid, "%u");

}

/**
 * gfs_plock_jhead_in - Read in plock jid-head data
 * @jh: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_plock_jhead_in(struct gfs_plock_jhead *jh, char *lvb)
{
	struct gfs_plock_jhead *str = (struct gfs_plock_jhead *) lvb;

	CPIN_32(jh, str, pj_magic);
	CPIN_16(jh, str, pj_lid);
	CPIN_16(jh, str, pj_jid);

}

/**
 * gfs_plock_jhead_out - Write out plock jid-head data
 * @jh: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_plock_jhead_out(struct gfs_plock_jhead *jh, char *lvb)
{
	struct gfs_plock_jhead *str = (struct gfs_plock_jhead *) lvb;

	CPOUT_32(jh, str, pj_magic);
	CPOUT_16(jh, str, pj_lid);
	CPOUT_16(jh, str, pj_jid);

}

/**
 * gfs_plock_jhead_print - print out jhead data
 * @ph: the cpu-order structure
 * @console - TRUE if this should be printed to the console,
 *            FALSE if it should be just printed to the incore debug
 *            buffer
 */

void
gfs_plock_jhead_print(struct gfs_plock_jhead *jh, int console)
{

	pv(jh, pj_magic, "%x");
	pv(jh, pj_lid, "%u");
	pv(jh, pj_jid, "%u");

}

/**
 * gfs_plock_elem_in - Read in plock element data
 * @el: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_plock_elem_in(struct gfs_plock_elem *el, char *lvb)
{
	struct gfs_plock_elem *str = (struct gfs_plock_elem *) lvb;

	CPIN_32(el, str, pe_magic);
	CPIN_32(el, str, pe_pid);
	CPIN_64(el, str, pe_start);
	CPIN_64(el, str, pe_end);
	CPIN_16(el, str, pe_excl);
	CPIN_16(el, str, pe_next_lid);
	CPIN_16(el, str, pe_mylid);

}

/**
 * gfs_plock_elem_out - Write out plock element data
 * @el: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_plock_elem_out(struct gfs_plock_elem *el, char *lvb)
{
	struct gfs_plock_elem *str = (struct gfs_plock_elem *) lvb;

	CPOUT_32(el, str, pe_magic);
	CPOUT_32(el, str, pe_pid);
	CPOUT_64(el, str, pe_start);
	CPOUT_64(el, str, pe_end);
	CPOUT_16(el, str, pe_excl);
	CPOUT_16(el, str, pe_next_lid);
	CPOUT_16(el, str, pe_mylid);

}

/**
 * gfs_plock_phead_print - print out phead data
 * @ph: the cpu-order structure
 * @console - TRUE if this should be printed to the console,
 *            FALSE if it should be just printed to the incore debug
 *            buffer
 */

void
gfs_plock_elem_print(struct gfs_plock_elem *el, int console)
{

	pv(el, pe_magic, "%x");
	pv(el, pe_pid, "%u");
	pv(el, pe_start, "%" PRIu64);
	pv(el, pe_end, "%" PRIu64);
	pv(el, pe_excl, "%u");
	pv(el, pe_next_lid, "%u");
	pv(el, pe_mylid, "%u");

}

/**
 * gfs_quota_lvb_in - Read in quota data
 * @rb: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_quota_lvb_in(struct gfs_quota_lvb *qb, char *lvb)
{
	struct gfs_quota_lvb *str = (struct gfs_quota_lvb *) lvb;

	CPIN_32(qb, str, qb_magic);
	CPIN_32(qb, str, qb_pad);
	CPIN_64(qb, str, qb_limit);
	CPIN_64(qb, str, qb_warn);
	CPIN_64(qb, str, qb_value);

}

/**
 * gfs_quota_lvb_out - Write out quota data
 * @rb: the cpu-order structure
 * @lvb: the lvb
 *
 */

void
gfs_quota_lvb_out(struct gfs_quota_lvb *qb, char *lvb)
{
	struct gfs_quota_lvb *str = (struct gfs_quota_lvb *) lvb;

	CPOUT_32(qb, str, qb_magic);
	CPOUT_32(qb, str, qb_pad);
	CPOUT_64(qb, str, qb_limit);
	CPOUT_64(qb, str, qb_warn);
	CPOUT_64(qb, str, qb_value);

}

/**
 * gfs_quota_lvb_print - Print out quota data
 * @rb: the cpu-order structure
 * @console - TRUE if this should be printed to the console,
 *            FALSE if it should be just printed to the incore debug
 *            buffer
 */

void
gfs_quota_lvb_print(struct gfs_quota_lvb *qb, int console)
{

	pv(qb, qb_magic, "%u");
	pv(qb, qb_pad, "%u");
	pv(qb, qb_limit, "%" PRIu64);
	pv(qb, qb_warn, "%" PRIu64);
	pv(qb, qb_value, "%" PRId64);

}
