/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#define SSTRING_SIZE 64
#define ALIAS_SIZE 35
#define CSTRING_SIZE 128
#define CSTRING_PRINT_SIZE 40
#define CSTRING "This string should be 41 characters long"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sisstring.h"

int main(int argc, char **argv)
{
        struct sstring *mystring, *myalias;
        char *str;
        int error = 0;

        str = (char *) malloc(sizeof(char) * CSTRING_SIZE);

        printf("Initializing a sstring of size %d...", SSTRING_SIZE);
        if((mystring = ss_init(SSTRING_SIZE)) == NULL) {
                printf("Failed.\n");
                return 1;
        }
        printf("ok.\n");

        printf("Initializing a sstring alias...");
        if((myalias = ss_init_alias()) == NULL) {
                printf("Failed.\n");
                return 1;
        }
        printf("ok.\n");

        printf("Character string is: %s\n", CSTRING);
        printf("Converting character string to a sstring...");
        if((error = cstring2ss(mystring, CSTRING, strlen(CSTRING))) != 0) {
                printf("Failed: %s\n", strerror(error));
                return 1;
        }
        printf("ok.\n");

        printf("Aliasing %d characters of sstring to sstring alias...",
               ALIAS_SIZE);
        if((error = ss_alias(myalias, mystring, 0, ALIAS_SIZE)) != 0) {
                printf("Failed: %s\n", strerror(error));
                return 1;
        }
        printf("ok.\n");

        printf("Copying from alias to character string of %d size...",
               CSTRING_PRINT_SIZE);
        if((error = ss2cstring(str, CSTRING_PRINT_SIZE, myalias)) != 0) {
                printf("Failed: %s\n", strerror(error));
                return 1;
        }
        printf("ok\n");

        printf("string: %s\n", str);

        printf("Attempting to convert a formatted set to sstring...");
        if((error = ss_sprintf(mystring, "test: %d %s\n", 10, "test2dsaf asdf as fasdfsadf asf asdfasdffffasdf as ")) != 0) {
                printf("Failed: %s\n", strerror(error));
                return 1;
        }
        printf("ok\n");

        printf("Copying from string to character string of %d size...",
               CSTRING_PRINT_SIZE);
        if((error = ss2cstring(str, CSTRING_PRINT_SIZE, mystring)) != 0) {
                printf("Failed: %s\n", strerror(error));
                return 1;
        }
        printf("ok\n");

        printf("string: %s\n", str);

        printf("Concatenating two sstrings...");
        if((error = ss_cat(mystring, myalias)) != 0) {
                printf("Failed: %s\n", strerror(error));
                return 1;
        }
        printf("ok\n");

        printf("Copying from string to character string of %d size...",
               CSTRING_PRINT_SIZE);
        if((error = ss2cstring(str, CSTRING_PRINT_SIZE, mystring)) != 0) {
                printf("Failed: %s\n", strerror(error));
                return 1;
        }
        printf("ok\n");

        printf("string: %s\n", str);

        printf("Destroying sstring...");
        if((error = ss_destroy(mystring)) != 0) {
                printf("Failed: %s\n", strerror(error));
                return 1;
        }
        printf("ok\n");

        printf("Destroying sstring alias...");
        if((error = ss_destroy(myalias)) != 0) {
                printf("Failed: %s\n", strerror(error));
                return 1;
        }
        printf("ok\n");

        return 0;

}
