/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

/* libpool_internal.h - * Internal libpool header file.
 * This should only be used when building libpool */

#ifndef __LIBPOOL_INTERNAL_H
#define __LIBPOOL_INTERNAL_H

#include <stdio.h>
#include "pool_ondisk.h"
#include "osi_list.h"

#define DEFAULT_MPSTRIPE 128
#define MAX_HARDCODED_MINOR 64

#define COMMDEV "/dev/poolcomm"

/* Fairly arbitrary size here, but it should be sufficient */
#define MAX_STRERROR 255

/* msg *must* be a null terminated character string */
#define SETMSGSS(s, msg) do { cstring2ss(s, msg, strlen(msg)); } while (0)

struct label_list {
  osi_list_t      ll_list;
  pool_label_t		ll_label;
  dev_t			   ll_dev;
  int             ll_sorted;
  int             ll_major;
  int             ll_minor;
  char           *ll_dev_path;
};


int get_module_version(uint64 *version);

/* 0 must be the last value - 0 cannot be a valid module version */
/* FIXME: add backwards compatibility */
static const uint64 valid_module_vers[] = {0x40103, 0};

int parse_pool_config(FILE *fp, struct label_list **lbls,
                      uint32 *subpool_count);

int set_pool_in_use(user_pool_t *p, struct sstring *emsg);

int set_pool_assembled(user_pool_t *p, int *assembled, struct sstring *emsg);

int get_blk_size(char *path, uint64 *blocks, struct sstring *emsg);

int scan_labels(struct label_list **l, struct sstring *emsg);

void destroy_labels(struct label_list *l);

void destroy_pl(user_pool_t *pl, int pool_count);

int check_poolname(char *pn);

int check_minornum(uint32 minor);

int get_pool_label(char *path, struct label_list **l);

#endif /*__LIBPOOL_INTERNAL_H */

/*
 * vim: et ts=3 sw=3 noai cin :
 */

