/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**  
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/
#ifndef __CCS_LOG_H__
#define __CCS_LOG_H__

#include <syslog.h>

/* Note, messages will always be sent to syslog, but userland programs **
** will have to close stdout, stderr if they don't want messages       **
** printed to the controling terminal (as all good daemon's should).   */

#ifdef DEBUG
#define log_dbg(fmt, args...) { \
    fprintf(stdout, "[%s:%d] " fmt , __FILE__ , __LINE__ , ## args ); \
    syslog(LOG_DEBUG, "[%s:%d] " fmt , __FILE__ , __LINE__ , ## args ); \
}
#else
#define log_dbg(fmt, args...)
#endif

#define log_msg(fmt, args...)  {\
    fprintf(stdout, fmt , ## args ); \
    syslog(LOG_NOTICE, fmt, ## args); \
}

#ifdef DEBUG
#define log_err(fmt, args...){ \
    fprintf(stderr, "[%s:%d] ", __FILE__ , __LINE__); \
    syslog(LOG_ERR, "[%s:%d] ", __FILE__ , __LINE__); \
    fprintf(stderr, fmt, ## args ); \
    syslog(LOG_ERR, fmt, ## args ); \
}
#else
#define log_err(fmt, args...){ \
    fprintf(stderr, fmt, ## args ); \
    syslog(LOG_ERR, fmt, ## args ); \
}
#endif


#ifdef DEBUG
#define log_sys_err(fmt, args...){ \
    fprintf(stderr, "[%s:%d] ", __FILE__ , __LINE__); \
    syslog(LOG_ERR, "[%s:%d] ", __FILE__ , __LINE__); \
    fprintf(stderr, fmt ": " , ## args ); \
    perror(NULL); \
    syslog(LOG_ERR, fmt ": %m\n" , ## args ); \
}
#else
#define log_sys_err(fmt, args...){ \
    fprintf(stderr, fmt ": " , ## args ); \
    perror(NULL); \
    syslog(LOG_ERR, fmt ": %m\n" , ## args ); \
}
#endif

#define die(ext, fmt, args...) { \
   fprintf(stderr, "In %s, at %d (%s) death by:\n" fmt , __FILE__ , \
         __LINE__ , GFS_RELEASE_NAME , ## args ); \
   syslog(LOG_ERR, "In %s, at %d (%s) death by:\n" fmt , __FILE__ , \
         __LINE__ , GFS_RELEASE_NAME , ## args ); exit(ext); \
}


#endif /* __CCS_LOG_H__ */
