/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**  
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/uio.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>  /* for rresvport */
#include <arpa/inet.h>
#include <asm/errno.h>
#include <errno.h>
#include "ccs_ft.h"


/*
 * _get_ccs_file
 * @filedata:
 * @filename:
 *
 * *filedata will be NULL if there is a failure.
 *
 * Returns: size of file on success, -EXXX on error
 */
int _get_ccs_file(char **filedata, char *filename){
  int first=1;
  int error=0;
  int sock_fd;
  int amount;
  int res_port = IPPORT_RESERVED-1;
  char buffer[512];
  char *data = NULL;
  int  data_size=0;
  ccs_fth_t header;
  struct sockaddr_in addr;
  struct iovec iov[2];
  long int addrinuse_timo=0;
  int connrefused_timo=1;

  sock_fd = socket(PF_INET, SOCK_STREAM, 0);

  if(sock_fd < 0){
    return -errno;
  }

  srandom(getpid());
  for(; res_port >= 512; res_port--){
    addr.sin_family = AF_INET;
    inet_aton("127.0.0.1", (struct in_addr *)&addr.sin_addr.s_addr);
    addr.sin_port = htons(res_port);
    if(bindresvport(sock_fd, &addr)){
      if(errno == EADDRINUSE){

	addrinuse_timo = random();
	addrinuse_timo /= (RAND_MAX/4);
	addrinuse_timo++;

	sleep(addrinuse_timo);

	continue;
      }
      return -errno;
    }

    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = INADDR_ANY;
    addr.sin_port = htons(CCS_DEFAULT_FEPORT);

    error = connect(sock_fd, (struct sockaddr *)&addr,
		    sizeof(struct sockaddr_in));

    if(error < 0){
      if(errno == ECONNREFUSED && connrefused_timo <= 8){
	/* server is probably overloaded.  Retry same port in a bit. */
	res_port++;
	sleep(connrefused_timo);
	connrefused_timo *= 2;
	continue;
      } else {
	close(sock_fd);
	return -errno;
      }
    } else {
      break;
    }
  }

  if(res_port < 512){
    close(sock_fd);
    return -errno;
  }

  iov[0].iov_base = &header;
  iov[0].iov_len  = sizeof(header);
  iov[1].iov_base = buffer;
  iov[1].iov_len  = sizeof(buffer);
  
  *filedata = NULL;

  /* Attention -- ok to send strlen +1? */
  error = write(sock_fd, filename, strlen(filename)+1);
  if(error < 0){
    close(sock_fd);
    return error;
  }

  while(1){
    memset(&header, 0, sizeof(header));

    amount = readv(sock_fd, iov, 2);
    if(amount < sizeof(header) || header.fth_magic != 0x101674 ||
       header.fth_error){
      if(amount < 0){
	error = amount;
      } else {
	error = -EBADE;
      }
      goto fail;
    }

    if(first && header.fth_last && !header.fth_count){
      error = -ENOENT;
      goto fail;
    }

    first=0;

    if((amount-sizeof(header)) != header.fth_count){
      /* Sometimes, we read everything that was on the wire, but it **
      ** isn't everything that was sent.  So, we try to finish the  **
      ** job here.  We put a limit on the number of retries and we  **
      ** don't tolerate read failures...                            */
      int rtn;
      int try_again=5;
      int index = amount-sizeof(header); /*header.fth_count;*/
      int diff = header.fth_count - (amount-sizeof(header));

      for(; try_again && diff; try_again--){
	rtn = read(sock_fd, &(buffer[index]), diff);
	if(rtn < 0){
	  error = rtn;
	  goto fail;
	}
	diff -= rtn;
	index += rtn;
      }
      
      if(diff){
	error = -EBADE;
	goto fail;
      }
    }

    if(!data){
      /* the plus 1 is so we can stick a NULL termination on the end **
      ** this is probably not the best way to fix a bug and will be  **
      ** revisited.................................................. */
      data = malloc(header.fth_count+1);
      if(!data){
	error = -ENOMEM;
	goto fail;
      }
      memcpy(data, buffer, header.fth_count);
      data_size = header.fth_count;
      if(header.fth_last) break;
    } else {
      char *tmp;
      /* the plus 1 is so we can stick a NULL termination on the end **
      ** this is probably not the best way to fix a bug and will be  **
      ** revisited.................................................. */
      tmp = realloc(data, (header.fth_count+data_size+1));
      if(!tmp){
	error = -ENOMEM;
	goto fail;
      }
      data = tmp;
      memcpy(data+data_size, buffer, header.fth_count);
      data_size += header.fth_count;
      if(header.fth_last){
	break;
      }
    }
    memset(buffer, 0, sizeof(buffer));
  }

  close(sock_fd);
  /* the plus 1 is so we can stick a NULL termination on the end **
  ** this is probably not the best way to fix a bug and will be  **
  ** revisited.................................................. */
  data[data_size++] = '\0';
  *filedata = data;
  return data_size;

 fail:
  if(data) free(data);
  close(sock_fd);
  *filedata = NULL;
  return error;
}


