/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#ifndef __gnbd_request_h__
#define __gnbd_request_h__

#ifndef __KERNEL__

typedef uint64_t u64;
typedef uint32_t u32;
typedef uint16_t u16;
typedef uint8_t u8;

#endif				/* __KERNEL */

struct create_req_s {
	char name[256];
	u32 dev_len;		/* includes null character */
	u32 timeout;
	u8 readonly;
};				/* the actual device name follows */
typedef struct create_req_s create_req_t;

#define CREATE_REQ_TO_BE(x) \
do { \
  (x)->dev_len = cpu_to_be32((x)->dev_len); \
  (x)->timeout = cpu_to_be32((x)->timeout); \
} while(0)

#define CREATE_REQ_TO_CPU(x) \
do {\
  (x)->dev_len = cpu_to_be32((x)->dev_len); \
  (x)->timeout = cpu_to_be32((x)->timeout); \
} while(0)

struct device_req_s {
	char name[256];
};
typedef struct device_req_s device_req_t;

typedef struct full_list_s {
	u64 devsize;
	u32 blksize;
	u32 timeout;
	u32 namelen;
	u32 len;
	u8 readonly;
} full_list_t;

#define FULL_LIST_TO_BE(x) \
do { \
  (x)->devsize = cpu_to_be64((x)->devsize); \
  (x)->blksize = cpu_to_be32((x)->blksize); \
  (x)->timeout = cpu_to_be32((x)->timeout); \
  (x)->len = cpu_to_be32((x)->len); \
} while(0)

#define FULL_LIST_TO_CPU(x) \
do { \
  (x)->devsize = be64_to_cpu((x)->devsize); \
  (x)->blksize = be32_to_cpu((x)->blksize); \
  (x)->timeout = be32_to_cpu((x)->timeout); \
  (x)->len = be32_to_cpu((x)->len); \
} while(0)

#define SERVER_NAME(x) ((char *)(x) + sizeof(full_list_t))

#define DEV_NAME(x) ((char *)(x) + sizeof(full_list_t) + (x)->namelen)

#define NEXT_SERVER(x) (full_list_t *)((char *)(x) + (x)->len)

#define GNBD_SERV_LOGIN_REQ         1
#define GNBD_SERV_FULL_LIST_REQ     2
#define GNBD_SERV_NAMES_LIST_REQ    3
#define GNBD_SERV_LIST_BANNED_REQ   4
#define GNBD_SERV_FENCE_REQ         5
#define GNBD_SERV_UNFENCE_REQ       6
#define GNBD_SERV_CREATE_REQ        7
#define GNBD_SERV_REMOVE_REQ        8
#define GNBD_SERV_FORCE_REMOVE_REQ  9
#define GNBD_SERV_TEST_ACTIVE_REQ  10
#define GNBD_SERV_KILL_GSERV       11

#define GNBD_SERV_SUCCESS_REPLY     0
/* the not gnbd specific failure values are simply their error code */

#define REPLY_ERR(x) (-((int)(x)))

#ifdef __KERNEL__

typedef struct request_s {
	struct task_struct *task;
	u32 client_ip;
	list_t list;
	long pid;
} request_t;

extern atomic_t request_count;

void init_gnbd_requestd(void);
int kill_requests(u32 client_ip);
int gnbd_requestd(void *args);

#define INC_REQUEST_COUNT atomic_inc(&request_count)
#define DEC_REQUEST_COUNT atomic_dec(&request_count)
#define REQUESTS_LEFT (atomic_read(&request_count))

#endif				/* __KERNEL__ */

#endif				/* __gnbd_request_h__ */
