/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#include "servd.h"
#include <linux/gnbd_request.h>
#include "trans.h"

struct task_struct *gnbd_servd_tsk = NULL;
u16 gnbd_servd_port;
int gnbd_servd_state;
struct completion gnbd_servd_wait;

int
gnbd_servd(void *arg)
{
	struct socket *sock = (struct socket *) arg;
	signal_info_t sig_info;
	int err;

	gnbd_daemonize("gnbd_servd");
	setup_blocking(&sig_info);
	gnbd_servd_tsk = current;
	complete(&gnbd_servd_wait);
	while (gnbd_servd_state != GNBD_SERVD_DEAD) {
		struct socket *newsock;

		if (accept_connection(sock, &newsock) < 0) {
			flush_signals(current);
			continue;
		}
		INC_REQUEST_COUNT;
		err = kernel_thread(gnbd_requestd, newsock, 0);
		if (err < 0) {
			DEC_REQUEST_COUNT;
			sock_release(newsock);
			printe("couldn't start gnbd_requestd kernel thread\n");
			continue;
		}
	}
	sock_release(sock);
	kill_requests(0);
	gnbd_servd_tsk = NULL;
	complete(&gnbd_servd_wait);
	return 0;
}

int
start_gnbd_servd(int port)
{
	struct socket *sock;
	int err;

	if (port < 0 || port > 65535) {
		printe("invalid port number (%d) for gnbd_servd\n", port);
		err = -EINVAL;
		goto fail;
	}
	if (port == 0)
		gnbd_servd_port = 14243;
	else
		gnbd_servd_port = (u16) port;

	gnbd_servd_state = GNBD_SERVD_ALIVE;
	init_completion(&gnbd_servd_wait);

	err = setup_server_sock(&gnbd_servd_port, &sock, 5, TRUE);
	if (err)
		goto fail;
	err = kernel_thread(gnbd_servd, sock, 0);
	if (err < 0) {
		printe("couldn't fork gnbd_servd kernel thread\n");
		goto fail_socket;
	}
	err = 0;

	wait_for_completion(&gnbd_servd_wait);

	return 0;

      fail_socket:
	sock_release(sock);
      fail:
	return err;
}

void
stop_gnbd_servd(void)
{
	int err;

	if (!gnbd_servd_tsk)
		printe("gnbd_servd is not running\n");
	else {
		gnbd_servd_state = GNBD_SERVD_DEAD;
		err = send_sig(SIGINT, gnbd_servd_tsk, 0);
		if (err)
			printe("error sending SIGINT to gnbd_servd (%d)\n",
			       err);
		wait_for_completion(&gnbd_servd_wait);
	}

	return;
}
